"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ...models.components import updaterunrequest as components_updaterunrequest
from ...models.components import updaterunresponse as components_updaterunresponse
from typing import Optional


@dataclasses.dataclass
class UpdateRunRequest:
    run_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'run_id', 'style': 'simple', 'explode': False }})
    update_run_request: components_updaterunrequest.UpdateRunRequest = dataclasses.field(metadata={'request': { 'media_type': 'application/json' }})
    



@dataclasses.dataclass
class UpdateRunResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: requests_http.Response = dataclasses.field()
    r"""Raw HTTP response; suitable for custom response parsing"""
    update_run_response: Optional[components_updaterunresponse.UpdateRunResponse] = dataclasses.field(default=None)
    r"""Successful response"""
    

